﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Ist.CS.CustomControl.FarPointCustomCellTypes
{
    public partial class MultiCheckBox : UserControl
    {
        public delegate void CheckBoxCheckedStateChangedEventHandler(int ItemID, int FlagID, bool Checked);
        public event CheckBoxCheckedStateChangedEventHandler CheckBoxCheckedStateChanged;

        private SortedList<int, CheckBox> mCheckBoxes = new SortedList<int, CheckBox>();  

        public MultiCheckBox()
        {
            InitializeComponent();
        }

        /// <summary>
        /// initialise control array & create checkboxes
        /// </summary>
        /// <param name="MultiCheckBoxItems"></param>
        /// <returns></returns>
        public bool Init(List<MultiCheckBoxItem> MultiCheckBoxItems)
        {
            int counter = 0;
            bool initSuccess = false;

            foreach (MultiCheckBoxItem multiCheckBoxItem in MultiCheckBoxItems)
            {
                CheckBox chkBox = new CheckBox();
                chkBox.AutoSize = false;
                chkBox.Text = multiCheckBoxItem.Text;
                chkBox.TabIndex = multiCheckBoxItem.ItemID;
                chkBox.Tag = multiCheckBoxItem.FlagID;
                chkBox.Checked = multiCheckBoxItem.Checked;
                chkBox.Height = 18;
                chkBox.Width = this.Width - 4;
                chkBox.Left = 3;
                chkBox.Top = counter * 16 + 3; // top gap = 3, control height with no padding = 16

                chkBox.Anchor = AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right;

                chkBox.CheckedChanged += new EventHandler(chkBox_CheckedChanged);

                this.Controls.Add(chkBox);

                counter++;

                try
                {
                    // add checkbox control to internal list
                    mCheckBoxes.Add(multiCheckBoxItem.FlagID, chkBox);
                    initSuccess = true;
                }
                catch (Exception Ex)
                {
                    //TODO: Error handling
                    initSuccess = false;
                }
            }
            // set height of control
            this.Height = mCheckBoxes.Count * 16 + (2 * 3);

            return initSuccess;
        }

        /// <summary>
        /// Event raised when the checked state of a single checkbox changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void chkBox_CheckedChanged(object sender, EventArgs e)
        {
            if (CheckBoxCheckedStateChanged != null)
            {
                CheckBoxCheckedStateChanged(((CheckBox) sender).TabIndex, 
                                            (int) ((CheckBox) sender).Tag,
                                            ((CheckBox) sender).Checked);
            }
        }

        /// <summary>
        /// Gets/Sets the composite value of all checkboxes. This is the sum of all checked checkboxes´ flag values.
        /// </summary>
        /// <returns></returns>
        public int CompositeValue
        {
            get
            {
                int compValue = 0;

                foreach (CheckBox checkBox in mCheckBoxes.Values)
                {
                    if (checkBox.Checked)
                    {
                        compValue = compValue + (int)checkBox.Tag;
                    }
                }

                return compValue;                
            }

            set
            {
                // check if a checkbox control is existing having a flagid contained in the given property value
                foreach (KeyValuePair<int, CheckBox> keyValuePair in mCheckBoxes)
                {
                    if ((value & keyValuePair.Key) == keyValuePair.Key)
                    {
                        keyValuePair.Value.Checked = true;
                    }
                    else
                    {
                        keyValuePair.Value.Checked = false;
                    }                    
                }
            }
        }

        /// <summary>
        /// Gets a comma seperated string containing the texts of all checked checkboxes
        /// </summary>
        /// <returns></returns>
        public string CompositeValueText
        {
            get
            {
                string compValueText = "";

                foreach (CheckBox checkBox in mCheckBoxes.Values)
                {
                    if (checkBox.Checked)
                    {
                        if (compValueText == "")
                        {
                            compValueText = checkBox.Text;    
                        }
                        else
                        {
                            compValueText = compValueText + ", " + checkBox.Text;    
                        }
                    }
                }
            
                return compValueText;                
            }
        }
    }

    public class MultiCheckBoxItem
    {
        public MultiCheckBoxItem(int ItemID, int FlagID, string Text, bool Checked)
        {
            this.ItemID = ItemID;
            this.FlagID = FlagID;
            this.Text = Text;
            this.Checked = Checked;
        }

        public int ItemID { get; set; }
        public int FlagID { get; set; }
        public string Text { get; set; }
        public bool Checked { get; set; }
    }
}
